@@ -15,9 +15,14 @@ project that is in Git.  A bridger of worlds, this plugin be.
 Try hg clone git:// or hg clone git+ssh://
 '''
 
+import inspect
 import os
 
-from mercurial import commands, extensions, hg, util as hgutil
+from mercurial import commands
+from mercurial import extensions
+from mercurial import hg
+from mercurial import localrepo
+from mercurial import util as hgutil
 from mercurial.i18n import _
 
 import gitrepo, hgrepo
@@ -92,6 +97,39 @@ def git_cleanup(ui, repo):
     map(f.write, new_map)
     ui.status(_('git commit map cleaned\n'))
 
+# drop this when we're 1.6-only, this just backports new behavior
+def sortednodetags(orig, *args, **kwargs):
+    ret = orig(*args, **kwargs)
+    ret.sort()
+    return ret
+extensions.wrapfunction(localrepo.localrepository, 'nodetags', sortednodetags)
+
+try:
+    from mercurial import discovery
+    kwname = 'heads'
+    if 'remoteheads' in inspect.getargspec(discovery.findoutgoing)[0]:
+        kwname = 'remoteheads'
+    def findoutgoing(orig, local, remote, *args, **kwargs):
+        kw = {}
+        kw.update(kwargs)
+        for val, k in zip(args, ('base', kwname, 'force')):
+            kw[k] = val
+        if isinstance(remote, gitrepo.gitrepo):
+            # clean up this cruft when we're 1.7-only, remoteheads and
+            # the return value change happened between 1.6 and 1.7.
+            git = GitHandler(local, local.ui)
+            base, heads = git.get_refs(remote.path)
+            newkw = {'base': base, kwname: heads}
+            newkw.update(kw)
+            kw = newkw
+            if kwname == 'heads':
+                r = orig(local, remote, **kw)
+                return [x[0] for x in r]
+        return orig(local, remote, **kw)
+    extensions.wrapfunction(discovery, 'findoutgoing', findoutgoing)
+except ImportError:
+    pass
+
 cmdtable = {
   "gimport":
         (gimport, [], _('hg gimport')),
